/***************************************************************************
*
* Program:  GenXFan.c		Author: S. Swift
*
* Date:  July 7, 2012		Version: 1.00
*
* This program implements a 2 ch fan controller with settable PWM duty cycle control
* and a single IO port configurable as either a digital input/output or Analog input.  
* The GenX board has optional external components that turn the analog port into
* a thermistor reader.
*
* Processor is an ATtiny45 running at 8MHz with debugwire enabled.  There is 
* a debugwire header on the GenX board.  Development was down using AVR Studio 4.19
* and a Atmel Dragon Emulator.  Output from the ATtiny45 controls two 3.5A Power Mosfets
* so there is plenty of power to drive fans or led strips.
*
* I2C address is 0x16.
*
* PWM control is assigned to port pins PB4 (OC1B) and PB1 (OC1A).
* Digital IO or analog input is on pin PB3 (ADC3).
* SCL is on pin PB2
* SDA is on pin PB0
*
* A simple protocol is used on the I2C line to handle communications with the 
* GenX ATmega1284P master.
*
* Protocol:
*
*  All commands use printable ASCII of four bytes or less.  Get commands also
*  return printable ASCII.
*
*    e - Both fans fully on
*    c - Both fans fully off
*    v - Return version - currently "V1.0"
*    g - Get digital or analog value  - gd or ga - 
*            digital returns ASCII 0 or 1
*            analog returns 10 bit analog binary value as four digits "0230"
*    s - Set digital or analog input/output direction - "sdi" or "sai"
*    f - Set selected fan fully on or off - "f1o"  or "f1f"
*    d - Set digital output to 0 or 1
*    p - Set PWM level for fan 1 or 2 - "p1128" or "p2034"
*
*  (c) Copyright 2012,  SOC Robotics, Inc.  
*
*  You may use, modify, enhance or change this program anyway you like.  Please change
*  the version number so our software doesn't get confused when requesting version
*  number using the v command. 
*
*  Enhancements:   PWM Ramp with settable delay - dimming function - interesting for 
*                  LED strings.
*
****************************************************************************/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include <util/delay.h>
#include "USI_TWI_Slave.h"


// Sample TWI transmission commands
#define TWI_CMD_MASTER_WRITE 0x10
#define TWI_CMD_MASTER_READ  0x20
#define TWI_CMD_MASTER_WRITE_ALOT 0x30

#define PWMCH1  '1'
#define PWMCH2	'2'
#define DIGITAL	'd'
#define ANALOG	'a'
#define VERSION 'v'
#define INPUT	'i'
#define OUTPUT	'o'
#define BITHIGH '1'
#define BITLOW	'0'
#define FAN1	'1'
#define	FAN2	'2'

#define SetBit(x,y) (x|=(1<<y)) 
#define ClrBit(x,y) (x&=~(1<<y)) 

char datamode,digitaldirection,analogdirection;
unsigned char location,location2,location3;
int analogin;

///////////////////////////////////////////////
// Software UART functions
//////////////////////////////////////////////

ISR(INT0_vect)
{

}

//Timer0 is general counter
void Init_Timer0(void)
{	 
	 TCNT0 = 0;
//	 TCCR0A = 0x00;
	TCCR0B = (1<<CS01); // clk/8
	// TCCR0A = (1<<WGM01);
	// TCCR0B = (1<<WGM01)|(1<<CS01);  // clk/8	 

}

//Timer1 is used to setup hardware PWM output on OC1A and OC1B
void Init_Timer1(void)
{
	TCNT1 = 0;
	// Reset cntr when OCR1C hit, PWM mode active, Set high on OC1A match,PCK/4
	//TCCR1 = (1<<CTC1)|(1<<PWM1A)|(1<<COM1A1)|(1<<COM1A0)|(1<<CS11)|(1<<CS10);
	//GTCCR = (1<<PWM1B)|(1<<COM1B1)|(1<<COM1B0);
	TCCR1 = (1<<CTC1)|(1<<PWM1A)|(1<<COM1A1)|(1<<CS11)|(1<<CS10);
	GTCCR = (1<<PWM1B)|(1<<COM1B1);
	OCR1A = 0;
	OCR1B = 0;
	OCR1C = 0xff;
}

void delayms (int delayTime)
{
//	delayTime /=8; //temporary measure for 1Mhz clock
	int i;
	for (i=0;i<delayTime*4;i++)
	{
		TIFR |= (1<<TOV0); //clear counter
		while ( !(TIFR & (1<<TOV0))) {}; //do nothing while waiting for counter overflow
	}	
}

// Initialize the two PWM fans outputs
//  PWM control is assigned to port pins PB4 (OC1B) and PB1 (OC1A).
void Init_PWM()
{
	// Set port direction and initial state
	DDRB |= (1<<PINB4)|(1<<PINB1);
	PORTB &= ~((1<<PINB4)|(1<<PINB1));

	// Setup the PWM registers by setting up timer
}

void Init_ADC()
{
	ADMUX = 0x03;
	ADCSRA = (1<<ADEN);
}

void Disable_ADC()
{
	ADCSRA = 0;  // Disable ADC
}

// Update the PWM duty cycle
void UpdatePWM(unsigned char pwmrate,char channel)
{
	if(channel==PWMCH1) {
		OCR1A = pwmrate;
	    }
	if(channel==PWMCH2) {
		OCR1B = pwmrate;
	    }
}

// Turn the fans off by setting duty cycle to 0
void Fan_On(char channel)
{
	if(channel==FAN1)  UpdatePWM(255,PWMCH1);
	if(channel==FAN2)  UpdatePWM(255,PWMCH2);
}

// Turn the fans off by setting duty cycle to 0
void Fan_Off(char channel)
{
	if(channel==FAN1)  UpdatePWM(0,PWMCH1);
	if(channel==FAN2)  UpdatePWM(0,PWMCH2);
}

// Setup the correct data acquisition mode
void SetDataMode(char mode)
{
	if(mode==DIGITAL) {
		if(digitaldirection==INPUT)  DDRB &= ~(1<<PINB3);
		else DDRB |= (1<<PINB3);
		Disable_ADC();
	    }
	if(mode==ANALOG) {
		Init_ADC();
	    }
}

int ReadADC()
{
	unsigned char highbyte,lowbyte;
	int analog;

	//  Start conversion
	ADCSRA |= (1<<ADSC);

	// Wait for  conversion to complete
	while(!(ADCSRA&(1<<ADIF))) ;

	lowbyte = ADCL;
	highbyte = ADCH;
	analog = highbyte*256 + lowbyte;

	return analog;
}

/* 
 * The program entry point. Initates TWI and enters eternal loop, waiting for data.
 */
int main( void )
{
	unsigned char pwmch1rate,pwmch2rate;
	char version[] = "V1.0";
	int thou,hun,tens,ones,rem,i;

	union {
	    unsigned char tempbuf[3];
		long int outint;
	} u2;

	volatile unsigned char TWI_slaveAddress, temp;
	unsigned char message[7];
	message[0]=0x00;
	message[1]=0x00;
	message[2]=0x00;
	message[3]=0x00;
	message[4]=0x00;
	message[5]=0x00;
	message[6]=0x00;

	location = 0;

//	Init_Timer0();
	// Set parameters
	datamode = ANALOG;
	digitaldirection = INPUT;
	analogdirection = INPUT;

	Init_Timer1();
	SetDataMode(ANALOG);
	Init_PWM();

  	// Own TWI slave address
  	TWI_slaveAddress = 0x16;
	USI_TWI_Slave_Initialise(TWI_slaveAddress);

 	sei();

	while(0) {
		SetBit(PORTB,PINB4);
		ClrBit(PORTB,PINB4);
		}

	Fan_Off(FAN1);
	Fan_Off(FAN2);

	//////////////////////////////////////////////////////////////
	// Start of main loop, depending on which communication method
	//////////////////////////////////////////////////////////////

	while(1)  {
		
		// See if any data received on TWI
		if(TWI_Get_Data_From_Transceiver(&message,5)) {
			switch(message[0]) {
				
				// Turn fan fully on or off
				case 'e':	
  					Fan_On(FAN1);
					Fan_On(FAN2);
				    break;
				case 'c':	
  					Fan_Off(FAN1);
					Fan_Off(FAN2);
				    break;

				case 'v':	
					message[0] = version[0];
					message[1] = version[1];
					message[2] = version[2];
					message[3] = version[3];
					TWI_Transceiver_Update_Data(message,4);
				    break;

				// Get analog or digital data
				case 'g':	
					if(datamode==DIGITAL) {
					    if((PINB&(1<<PINB3)))  message[0] = '1';
						else message[0] = '0';
						TWI_Transceiver_Update_Data(message,1);
						}
					// Return analog input value
					else if(datamode==ANALOG) {
						analogin = ReadADC();
						//analogin = 567;
						thou = analogin/1000;
						rem = analogin-thou*1000;
						hun = rem/100;
						rem = rem - hun*100;
						tens = rem/10;
						ones = rem - tens*10;
						message[0] = (unsigned char) thou + '0';
						message[1] = (unsigned char) hun + '0';
						message[2] = (unsigned char) tens + '0';
						message[3] = (unsigned char) ones + '0';
						TWI_Transceiver_Update_Data(message,4);
						}
				    break;
				
				// Set data acquisition mode - digital or analog
				// For digital 'i' - input, 'o' - output
				// Analog direction doesn't apply for ATtiny45
		        case 's':
					datamode = message[1];
					if(datamode==DIGITAL) {
						digitaldirection = message[2];
					    SetDataMode(DIGITAL);
						}
					if(datamode==ANALOG) {
						analogdirection = message[2];
					    SetDataMode(ANALOG);
						}

				    break;

				// Set fan fully on or off
		        case 'f':
					if(message[1]==FAN1) {
						if(message[2]=='o')  Fan_On(FAN1);
						if(message[2]=='f')  Fan_Off(FAN1);
					    }
					else if(message[1]==FAN2) {
						if(message[2]=='o')  Fan_On(FAN2);
						if(message[2]=='f')  Fan_Off(FAN2);
					    }
				    break;

				// Set digital output level
		        case 'd':
					if(digitaldirection==OUTPUT) {
					    if(message[1]==BITHIGH) SetBit(PORTB,PINB3);
						if(message[1]==BITLOW) ClrBit(PORTB,PINB3);
						}
				    break;

				case 'p':
					// Set PWM channel 1 - handle leading space characters
			        if(message[1]=='1') {
					    if(message[4]==' ') pwmch1rate = 0;
						else pwmch1rate = (message[4]-'0');
						if(message[3]==' ') ;
						else pwmch1rate += (message[3]-'0')*10;
						if(message[2]==' ') ;
						else pwmch1rate += (message[2]-'0')*100;
						UpdatePWM(pwmch1rate,PWMCH1);
						} 

					// Set PWM channel 1
					if(message[1]=='2') {
					    if(message[4]==' ') pwmch1rate = 0;
						else pwmch2rate = (message[4]-'0');
						if(message[3]==' ') ;
						else pwmch2rate += (message[3]-'0')*10;
						if(message[3]==' ') ;
						else pwmch2rate += (message[2]-'0')*100;
						UpdatePWM(pwmch2rate,PWMCH2);
						}
					break;
					}
				}
				
			}	
}

